@@ -106,6 +106,8 @@ rb_gst_media_type_to_extension (const char *media_type)
 		return "mp3";
 	} else if (!strcmp (media_type, "audio/x-vorbis") || !strcmp (media_type, "application/ogg")) {
 		return "ogg";
+	} else if (!strcmp (media_type, "audio/x-opus")) {
+		return "opus";
 	} else if (!strcmp (media_type, "audio/x-flac") || !strcmp (media_type, "audio/flac")) {
 		return "flac";
 	} else if (!strcmp (media_type, "audio/x-aac") || !strcmp (media_type, "audio/aac") || !strcmp (media_type, "audio/x-alac")) {
@@ -198,7 +200,7 @@ rb_gst_encoding_profile_get_media_type (GstEncodingProfile *profile)
 }
 
 GstEncodingTarget *
-rb_gst_get_default_encoding_target ()
+rb_gst_get_default_encoding_target (void)
 {
 	if (default_target == NULL) {
 		char *target_file;
