@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Ross Burton <ross@burtonini.com>
  */
@@ -272,17 +271,25 @@ build_encoder (SjExtractor *extractor)
 {
   SjExtractorPrivate *priv;
   GstElement *encodebin;
+  const char *profile_name;
+  static const char * mp3_pipeline = "lamemp3enc ! xingmux ! id3v2mux";
 
   g_return_val_if_fail (SJ_IS_EXTRACTOR (extractor), NULL);
   priv = (SjExtractorPrivate*)extractor->priv;
   g_return_val_if_fail (priv->profile != NULL, NULL);
 
-  encodebin = gst_element_factory_make ("encodebin", NULL);
-  if (encodebin == NULL)
-    return NULL;
-  g_object_set (encodebin, "profile", priv->profile, NULL);
-  /* Nice big buffers... */
-  g_object_set (encodebin, "queue-time-max", 120 * GST_SECOND, NULL);
+  /* encodebin does not use xingmux so do mp3 pipeline ourselves */
+  profile_name = gst_encoding_profile_get_name (priv->profile);
+  if (strcmp (profile_name, "mp3") == 0) {
+    encodebin = gst_parse_bin_from_description (mp3_pipeline, TRUE, NULL);
+  } else {
+    encodebin = gst_element_factory_make ("encodebin", NULL);
+    if (encodebin == NULL)
+      return NULL;
+    g_object_set (encodebin, "profile", priv->profile, NULL);
+    /* Nice big buffers... */
+    g_object_set (encodebin, "queue-time-max", 120 * GST_SECOND, NULL);
+  }
 
   return encodebin;
 }
@@ -523,6 +530,20 @@ sj_extractor_extract_track (SjExtractor *extractor, const TrackDetails *track, G
                                GST_TAG_DURATION, track->duration * GST_SECOND,
                                NULL);
 
+     if (track->composer != NULL && strcmp (track->composer, "") != 0) {
+        gst_tag_setter_add_tags (tagger,
+                            GST_TAG_MERGE_APPEND,
+                            GST_TAG_COMPOSER, track->composer,
+                            NULL);
+      }
+
+     if (track->composer_sortname != NULL && strcmp (track->composer_sortname, "") != 0) {
+        gst_tag_setter_add_tags (tagger,
+                            GST_TAG_MERGE_APPEND,
+                            GST_TAG_COMPOSER_SORTNAME, track->composer_sortname,
+                            NULL);
+      }
+
       if (track->album->album_id != NULL && strcmp (track->album->album_id, "") != 0) {
         gst_tag_setter_add_tags (tagger,
                             GST_TAG_MERGE_APPEND,
@@ -684,5 +705,21 @@ gboolean
 sj_extractor_supports_profile (GstEncodingProfile *profile)
 {
   /* TODO: take a GError to return a message if the profile isn't supported */
+  const gchar *profile_name = gst_encoding_profile_get_name (profile);
+  if (strcmp (profile_name, "mp3") == 0) {
+    GstElementFactory *factory = gst_element_factory_find ("lamemp3enc");
+    if (factory == NULL)
+      return FALSE;
+    g_object_unref (factory);
+    factory = gst_element_factory_find ("xingmux");
+    if (factory == NULL)
+      return FALSE;
+    g_object_unref (factory);
+    factory = gst_element_factory_find ("id3v2mux");
+    if (factory == NULL)
+      return FALSE;
+    g_object_unref (factory);
+    return TRUE;
+  }
   return !rb_gst_check_missing_plugins(profile, NULL, NULL);
 }
