@@ -15,9 +15,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +26,6 @@
 #include <stdlib.h>
 #include <glib.h>
 #include <glib-object.h>
-#include <gconf/gconf-client.h>
 #include <discid/discid.h>
 #include <musicbrainz5/mb5_c.h>
 
@@ -36,6 +33,8 @@
 #include "sj-structures.h"
 #include "sj-error.h"
 
+static char language[3];
+
 #define GET(field, function, obj) {						\
 	function (obj, buffer, sizeof (buffer));				\
 	if (field)								\
@@ -46,22 +45,23 @@
 		field = g_strdup (buffer);					\
 }
 
-#define GCONF_MUSICBRAINZ_SERVER "/apps/sound-juicer/musicbrainz_server"
-#define GCONF_PROXY_USE_PROXY "/system/http_proxy/use_http_proxy"
-#define GCONF_PROXY_HOST "/system/http_proxy/host"
-#define GCONF_PROXY_PORT "/system/http_proxy/port"
-#define GCONF_PROXY_USE_AUTHENTICATION "/system/http_proxy/use_authentication"
-#define GCONF_PROXY_USERNAME "/system/http_proxy/authentication_user"
-#define GCONF_PROXY_PASSWORD "/system/http_proxy/authentication_password"
+#ifndef DISCID_HAVE_SPARSE_READ
+#define discid_read_sparse(disc, dev, i) discid_read(disc, dev)
+#endif
+
 #define SJ_MUSICBRAINZ_USER_AGENT "libjuicer-"VERSION
 
 typedef struct {
   Mb5Query mb;
-  DiscId *disc;
-  char *cdrom;
+  DiscId  *disc;
+  char    *cdrom;
+  GHashTable *artist_cache;
   /* Proxy */
-  char *http_proxy;
-  int http_proxy_port;
+  char    *proxy_host;
+  char    *proxy_username;
+  char    *proxy_password;
+  int      proxy_port;
+  gboolean proxy_use_authentication;
 } SjMetadataMusicbrainz5Private;
 
 #define GET_PRIVATE(o)  \
@@ -70,9 +70,11 @@ typedef struct {
 enum {
   PROP_0,
   PROP_DEVICE,
-  PROP_USE_PROXY,
+  PROP_PROXY_USE_AUTHENTICATION,
   PROP_PROXY_HOST,
   PROP_PROXY_PORT,
+  PROP_PROXY_USERNAME,
+  PROP_PROXY_PASSWORD
 };
 
 static void metadata_interface_init (gpointer g_iface, gpointer iface_data);
@@ -102,8 +104,131 @@ sj_mb5_album_details_dump (AlbumDetails *details)
 #define sj_mb5_album_details_dump(...)
 #endif
 
+
+static void
+parse_artist_aliases (Mb5Artist   artist,
+                      char      **name,
+                      char      **sortname)
+{
+  Mb5AliasList alias_list;
+  int i;
+  char buffer[512]; /* for the GET macro */
+  char locale[512];
+
+  GET (*name, mb5_artist_get_name, artist);
+  GET (*sortname, mb5_artist_get_sortname, artist);
+
+  if (*language == 0)
+    return;
+
+  alias_list = mb5_artist_get_aliaslist (artist);
+  if (alias_list == NULL)
+    return;
+
+  for (i = 0; i < mb5_alias_list_size (alias_list); i++) {
+    Mb5Alias alias = mb5_alias_list_item (alias_list, i);
+
+    if (alias == NULL)
+      continue;
+
+    if (mb5_alias_get_locale (alias, locale, sizeof(locale)) > 0 &&
+        strcmp (locale, language) == 0 &&
+        mb5_alias_get_primary (alias, buffer, sizeof(buffer)) > 0 &&
+        strcmp (buffer, "primary") == 0) {
+      GET (*name, mb5_alias_get_text, alias);
+      GET (*sortname, mb5_alias_get_sortname, alias);
+    }
+  }
+}
+
+static ArtistDetails*
+make_artist_details (SjMetadataMusicbrainz5 *self,
+                     Mb5Artist               artist)
+{
+  char buffer[512]; /* For the GET macro */
+  ArtistDetails *details;
+  SjMetadataMusicbrainz5Private *priv = GET_PRIVATE (self);
+
+  mb5_artist_get_id (artist, buffer, sizeof(buffer));
+  details = g_hash_table_lookup (priv->artist_cache, buffer);
+  if (details == NULL) {
+    details = g_new0 (ArtistDetails, 1);
+    details->id = g_strdup (buffer);
+    parse_artist_aliases (artist, &details->name, &details->sortname);
+    GET (details->disambiguation, mb5_artist_get_disambiguation, artist);
+    GET (details->gender, mb5_artist_get_gender, artist);
+    GET (details->country, mb5_artist_get_country, artist);
+    g_hash_table_insert (priv->artist_cache, details->id, details);
+  }
+  return details;
+}
+
+static void
+print_musicbrainz_query_error (SjMetadataMusicbrainz5 *self,
+                               const char             *entity,
+                               const char             *id)
+{
+  SjMetadataMusicbrainz5Private *priv = GET_PRIVATE (self);
+  int len;
+  char *message;
+  int code =  mb5_query_get_lasthttpcode (priv->mb);
+  /* No error if the discid isn't found */
+  if (strcmp (entity, "discid") == 0 && code == 404)
+    return;
+  len =  mb5_query_get_lasterrormessage (priv->mb, NULL, 0) + 1;
+  message = g_malloc (len);
+  mb5_query_get_lasterrormessage (priv->mb, message, len);
+  g_warning ("No Musicbrainz metadata for %s %s, http code %d, %s",
+             entity, id, code, message);
+  g_free (message);
+}
+
+
+static Mb5Metadata
+query_musicbrainz (SjMetadataMusicbrainz5 *self,
+                   const char             *entity,
+                   const char             *id,
+                   char                   *includes)
+{
+  Mb5Metadata metadata;
+  char *inc[] = { "inc" };
+  SjMetadataMusicbrainz5Private *priv = GET_PRIVATE (self);
+
+  if (includes == NULL)
+    metadata = mb5_query_query (priv->mb, entity, id, "",
+                                0, NULL, NULL);
+  else
+    metadata = mb5_query_query (priv->mb, entity, id, "",
+                                1, inc, &includes);
+  if (metadata == NULL)
+    print_musicbrainz_query_error (self, entity, id);
+
+  return metadata;
+}
+
+static ArtistDetails*
+query_artist (SjMetadataMusicbrainz5 *self,
+              const gchar            *id)
+{
+  SjMetadataMusicbrainz5Private *priv = GET_PRIVATE (self);
+  ArtistDetails *details = g_hash_table_lookup (priv->artist_cache, id);
+
+  if (details == NULL) {
+    Mb5Artist artist;
+    Mb5Metadata metadata = query_musicbrainz (self, "artist", id, "aliases");
+    if (metadata == NULL)
+      return NULL;
+    artist = mb5_metadata_get_artist (metadata);
+    if (artist != NULL)
+      details = make_artist_details (self, artist);
+    mb5_metadata_delete (metadata);
+  }
+  return details;
+}
+
 static GList *
-get_artist_list (Mb5ArtistCredit credit)
+get_artist_list (SjMetadataMusicbrainz5 *self,
+                 Mb5ArtistCredit         credit)
 {
   Mb5NameCreditList name_list;
   GList *artists;
@@ -122,25 +247,19 @@ get_artist_list (Mb5ArtistCredit credit)
   for (i = 0; i < mb5_namecredit_list_size (name_list); i++) {
     Mb5NameCredit name_credit;
     Mb5Artist artist;
-    ArtistDetails *details;
+    ArtistCredit *artist_credit = g_slice_new0 (ArtistCredit);
 
     name_credit = mb5_namecredit_list_item (name_list, i);
-    details = g_new0 (ArtistDetails, 1);
-    GET (details->joinphrase, mb5_namecredit_get_joinphrase, name_credit);
-    artists = g_list_prepend (artists, details);
     artist = mb5_namecredit_get_artist (name_credit);
-    if (!artist) {
+    if (artist != NULL) {
+      artist_credit->details = make_artist_details (self, artist);
+    } else {
       g_warning ("no Mb5Artist associated with Mb5NameCredit, falling back to Mb5NameCredit::name");
-      GET (details->name, mb5_namecredit_get_name, name_credit);
-      continue;
+      artist_credit->details = g_new0 (ArtistDetails, 1);
+      GET (artist_credit->details->name, mb5_namecredit_get_name, name_credit);
     }
-
-    GET (details->id, mb5_artist_get_id, artist);
-    GET (details->name, mb5_artist_get_name, artist);
-    GET (details->sortname, mb5_artist_get_sortname, artist);
-    GET (details->disambiguation, mb5_artist_get_disambiguation, artist);
-    GET (details->gender, mb5_artist_get_gender, artist);
-    GET (details->country, mb5_artist_get_country, artist);
+    GET (artist_credit->joinphrase, mb5_namecredit_get_joinphrase, name_credit);
+    artists = g_list_prepend (artists, artist_credit);
   }
 
   return g_list_reverse(artists);
@@ -156,11 +275,11 @@ get_artist_info (GList *artists, char **name, char **sortname, char **id)
   artist_name = g_string_new (NULL);
   artist_count = 0;
   for (it = artists; it != NULL; it = it->next) {
-    ArtistDetails *details = (ArtistDetails *)it->data;
+    ArtistCredit *artist_credit = (ArtistCredit *)it->data;
     artist_count++;
-    g_string_append (artist_name, details->name);
-    if (details->joinphrase != NULL)
-      g_string_append (artist_name, details->joinphrase);
+    g_string_append (artist_name, artist_credit->details->name);
+    if (artist_credit->joinphrase != NULL)
+      g_string_append (artist_name, artist_credit->joinphrase);
   }
 
   if (artist_count != 1) {
@@ -170,7 +289,7 @@ get_artist_info (GList *artists, char **name, char **sortname, char **id)
       if (id != NULL)
         *id = NULL;
   } else {
-      ArtistDetails *details = (ArtistDetails *)artists->data;
+      ArtistDetails *details = ((ArtistCredit *)artists->data)->details;
       if (sortname != NULL)
         *sortname = g_strdup (details->sortname);
       if (id != NULL)
@@ -183,6 +302,40 @@ get_artist_info (GList *artists, char **name, char **sortname, char **id)
   g_string_free (artist_name, FALSE);
 }
 
+static GList*
+get_release_labels (Mb5Release *release)
+{
+  Mb5LabelInfoList list;
+  int i;
+  char buffer[512]; /* for the GET() macro */
+  GList *label_list = NULL;
+
+  list = mb5_release_get_labelinfolist (release);
+  if (list == NULL)
+    return NULL;
+
+  for (i = 0; i < mb5_labelinfo_list_size (list); i++) {
+    Mb5LabelInfo info;
+    Mb5Label label;
+    LabelDetails *label_data;
+
+    info = mb5_labelinfo_list_item (list, i);
+    if (info == NULL)
+      continue;
+
+    label = mb5_labelinfo_get_label (info);
+    if (label == NULL)
+      continue;
+
+    label_data = g_new0 (LabelDetails, 1);
+    GET (label_data->name,     mb5_label_get_name,     label);
+    GET (label_data->sortname, mb5_label_get_sortname, label);
+    label_list = g_list_prepend (label_list, label_data);
+  }
+  label_list = g_list_reverse (label_list);
+  return label_list;
+}
+
 static void
 fill_album_composer (AlbumDetails *album)
 {
@@ -220,13 +373,14 @@ fill_album_composer (AlbumDetails *album)
   }
 }
 
-typedef void (*RelationForeachFunc)(Mb5Relation relation, gpointer user_data);
+typedef void (*RelationForeachFunc)(SjMetadataMusicbrainz5 *self, Mb5Relation relation, gpointer user_data);
 
-static void relationlist_list_foreach_relation(Mb5RelationListList relation_lists,
-                                               const char *targettype,
-                                               const char *relation_type,
-                                               RelationForeachFunc callback,
-                                               gpointer user_data)
+static void relationlist_list_foreach_relation(SjMetadataMusicbrainz5 *self,
+                                               Mb5RelationListList     relation_lists,
+                                               const char             *targettype,
+                                               const char             *relation_type,
+                                               RelationForeachFunc     callback,
+                                               gpointer                user_data)
 {
   unsigned int j;
 
@@ -257,26 +411,40 @@ static void relationlist_list_foreach_relation(Mb5RelationListList relation_list
       if (!g_str_equal (type, relation_type))
         continue;
 
-      callback(relation, user_data);
+      callback(self, relation, user_data);
     }
   }
 }
 
-static void composer_cb (Mb5Relation relation, gpointer user_data)
+static void composer_cb (SjMetadataMusicbrainz5 *self,
+                         Mb5Relation             relation,
+                         gpointer                user_data)
 {
-      Mb5Artist composer;
-      TrackDetails *track = (TrackDetails *)user_data;
-      char buffer[512]; /* for the GET() macro */
+  Mb5Artist composer;
+  TrackDetails *track = (TrackDetails *)user_data;
+  char buffer[512]; /* For the GET macro */
+  ArtistDetails *details;
 
-      composer = mb5_relation_get_artist (relation);
-      if (composer == NULL)
-        return;
-
-      GET (track->composer, mb5_artist_get_name, composer);
-      GET (track->composer_sortname, mb5_artist_get_sortname, composer);
+  composer = mb5_relation_get_artist (relation);
+  if (composer == NULL)
+    return;
+  /* NB work-level-rels only returns artist name, sortname & id so
+     we need to perform another query if we want the alias list
+     therefore use query_artist rather than make_artist_details. */
+  mb5_artist_get_id (composer, buffer, sizeof (buffer));
+  details = query_artist (self, buffer);
+  if (details != NULL) {
+    track->composer = g_strdup (details->name);
+    track->composer_sortname = g_strdup (details->sortname);
+  } else {
+    GET (track->composer, mb5_artist_get_name, composer);
+    GET (track->composer_sortname, mb5_artist_get_sortname, composer);
+  }
 }
 
-static void work_cb (Mb5Relation relation, gpointer user_data)
+static void work_cb (SjMetadataMusicbrainz5 *self,
+                     Mb5Relation             relation,
+                     gpointer                user_data)
 {
     Mb5RelationListList relation_lists;
     Mb5Work work;
@@ -285,22 +453,45 @@ static void work_cb (Mb5Relation relation, gpointer user_data)
     if (work == NULL)
         return;
     relation_lists = mb5_work_get_relationlistlist (work);
-    relationlist_list_foreach_relation (relation_lists, "artist", "composer",
+    relationlist_list_foreach_relation (self, relation_lists,
+                                        "artist", "composer",
                                         composer_cb, user_data);
 }
 
 static void
-fill_recording_relations (Mb5Recording recording, TrackDetails *track)
+fill_recording_relations (SjMetadataMusicbrainz5 *self,
+                          Mb5Recording            recording,
+                          TrackDetails           *track)
 {
   Mb5RelationListList relation_lists;
 
   relation_lists = mb5_recording_get_relationlistlist (recording);
-  relationlist_list_foreach_relation(relation_lists, "work", "performance",
+  relationlist_list_foreach_relation(self, relation_lists,
+                                     "work", "performance",
                                      work_cb, track);
 }
 
 static void
-fill_tracks_from_medium (Mb5Medium medium, AlbumDetails *album)
+parse_artist_credit (SjMetadataMusicbrainz5  *self,
+                     Mb5ArtistCredit          credit,
+                     char                   **name,
+                     char                   **sortname,
+                     char                   **id)
+{
+  if (credit) {
+    GList *artists;
+    artists = get_artist_list (self, credit);
+    if (artists) {
+      get_artist_info (artists, name, sortname, id);
+      g_list_free_full (artists, artist_credit_destroy);
+    }
+  }
+}
+
+static void
+fill_tracks_from_medium (SjMetadataMusicbrainz5 *self,
+                         Mb5Medium               medium,
+                         AlbumDetails           *album)
 {
   Mb5TrackList track_list;
   GList *tracks;
@@ -331,32 +522,30 @@ fill_tracks_from_medium (Mb5Medium medium, AlbumDetails *album)
 
     track->number = mb5_track_get_position (mbt);
 
-    /* duration from track is more reliable than from recording */
+    /* Prefer metadata from Mb5Track over metadata from Mb5Recording
+     * https://bugzilla.gnome.org/show_bug.cgi?id=690903#c8
+     */
     track->duration = mb5_track_get_length (mbt) / 1000;
+    GET (track->title, mb5_track_get_title, mbt);
+    credit = mb5_track_get_artistcredit (mbt);
 
     recording = mb5_track_get_recording (mbt);
     if (recording != NULL) {
-      GET (track->title, mb5_recording_get_title, recording);
       GET (track->track_id, mb5_recording_get_id, recording);
-      fill_recording_relations (recording, track);
+      fill_recording_relations (self, recording, track);
+
       if (track->duration == 0)
         track->duration = mb5_recording_get_length (recording) / 1000;
-      credit = mb5_recording_get_artistcredit (recording);
-    } else {
-      GET (track->title, mb5_track_get_title, mbt);
-      credit = mb5_track_get_artistcredit (mbt);
+      if (track->title == NULL)
+        GET (track->title, mb5_recording_get_title, recording);
+      if (credit == NULL)
+          credit = mb5_recording_get_artistcredit (recording);
     }
 
-    if (credit) {
-      GList *artists;
-      artists = get_artist_list (credit);
-      if (artists) {
-        get_artist_info (artists, &track->artist,
-                         &track->artist_sortname,
-                         &track->artist_id);
-      }
-      track->artists = artists;
-    }
+      parse_artist_credit(self, credit,
+                          &track->artist,
+                          &track->artist_sortname,
+                          &track->artist_id);
     if (track->artist == NULL)
       track->artist = g_strdup (album->artist);
     if (track->artist_sortname == NULL)
@@ -369,7 +558,9 @@ fill_tracks_from_medium (Mb5Medium medium, AlbumDetails *album)
   album->tracks = g_list_reverse (tracks);
 }
 
-static void wikipedia_cb (Mb5Relation relation, gpointer user_data)
+static void wikipedia_cb (SjMetadataMusicbrainz5 *self,
+                          Mb5Relation             relation,
+                          gpointer                user_data)
 {
   AlbumDetails *album = (AlbumDetails *)user_data;
   char buffer[512]; /* for the GET() macro */
@@ -382,7 +573,9 @@ static void wikipedia_cb (Mb5Relation relation, gpointer user_data)
   }
 }
 
-static void discogs_cb (Mb5Relation relation, gpointer user_data)
+static void discogs_cb (SjMetadataMusicbrainz5 *self,
+                        Mb5Relation             relation,
+                        gpointer                user_data)
 {
   AlbumDetails *album = (AlbumDetails *)user_data;
   char buffer[512]; /* for the GET() macro */
@@ -395,7 +588,9 @@ static void discogs_cb (Mb5Relation relation, gpointer user_data)
   }
 }
 
-static void lyrics_cb (Mb5Relation relation, gpointer user_data)
+static void lyrics_cb (SjMetadataMusicbrainz5 *self,
+                       Mb5Relation             relation,
+                       gpointer                user_data)
 {
   AlbumDetails *album = (AlbumDetails *)user_data;
   char buffer[512]; /* for the GET() macro */
@@ -409,25 +604,31 @@ static void lyrics_cb (Mb5Relation relation, gpointer user_data)
 }
 
 static void
-fill_relations (Mb5RelationListList relation_lists, AlbumDetails *album)
+fill_relations (SjMetadataMusicbrainz5 *self,
+                Mb5RelationListList     relation_lists,
+                AlbumDetails           *album)
 {
-  relationlist_list_foreach_relation (relation_lists, "url", "wikipedia",
+  relationlist_list_foreach_relation (self, relation_lists,
+                                      "url", "wikipedia",
                                       wikipedia_cb, album);
-  relationlist_list_foreach_relation (relation_lists, "url", "discogs",
+  relationlist_list_foreach_relation (self, relation_lists,
+                                      "url", "discogs",
                                       discogs_cb, album);
-  relationlist_list_foreach_relation (relation_lists, "url", "lyrics",
+  relationlist_list_foreach_relation (self, relation_lists,
+                                      "url", "lyrics",
                                       lyrics_cb, album);
 }
 
 static AlbumDetails *
-make_album_from_release (Mb5ReleaseGroup group,
-                         Mb5Release release,
-                         Mb5Medium medium)
+make_album_from_release (SjMetadataMusicbrainz5 *self,
+                         Mb5ReleaseGroup         group,
+                         Mb5Release              release,
+                         Mb5Medium               medium)
 {
   AlbumDetails *album;
   Mb5ArtistCredit credit;
   Mb5RelationListList relationlists;
-  GList *artists;
+  Mb5MediumList media;
   char *date = NULL;
   char buffer[512]; /* for the GET macro */
 
@@ -436,6 +637,10 @@ make_album_from_release (Mb5ReleaseGroup group,
 
   album = g_new0 (AlbumDetails, 1);
 
+  media = mb5_release_get_mediumlist (release);
+  if (media)
+    album->disc_count = mb5_medium_list_size (media);
+
   GET (album->album_id, mb5_release_get_id, release);
   GET (album->title, mb5_medium_get_title, medium);
   if (album->title == NULL)
@@ -443,20 +648,18 @@ make_album_from_release (Mb5ReleaseGroup group,
 
   credit = mb5_release_get_artistcredit (release);
 
-  artists = get_artist_list (credit);
-  if (artists) {
-    get_artist_info (artists, &album->artist,
-                     &album->artist_sortname,
-                     &album->artist_id);
-  }
-  album->artists = artists;
+  parse_artist_credit (self, credit,
+                       &album->artist,
+                       &album->artist_sortname,
+                       &album->artist_id);
 
   GET (date, mb5_release_get_date, release);
   album->release_date = sj_metadata_helper_scan_date (date);
   g_free (date);
 
   GET (album->asin, mb5_release_get_asin, release);
-  GET (album->country, mb5_release_get_country, release);
+  mb5_release_get_country (release, buffer, sizeof(buffer));
+  album->country = sj_metadata_helper_lookup_country_code (buffer);
   if (group) {
     GET (album->type, mb5_releasegroup_get_primarytype, group);
     if (g_str_has_suffix (album->type, "Spokenword")
@@ -465,14 +668,15 @@ make_album_from_release (Mb5ReleaseGroup group,
       album->is_spoken_word = TRUE;
     }
     relationlists = mb5_releasegroup_get_relationlistlist (group);
-    fill_relations (relationlists, album);
+    fill_relations (self, relationlists, album);
   }
 
   album->disc_number = mb5_medium_get_position (medium);
-  fill_tracks_from_medium (medium, album);
+  fill_tracks_from_medium (self, medium, album);
   fill_album_composer (album);
   relationlists = mb5_release_get_relationlistlist (release);
-  fill_relations (relationlists, album);
+  fill_relations (self, relationlists, album);
+  album->labels = get_release_labels (release);
 
   sj_mb5_album_details_dump (album);
   return album;
@@ -484,6 +688,7 @@ make_album_from_release (Mb5ReleaseGroup group,
 static GList *
 mb5_list_albums (SjMetadata *metadata, char **url, GError **error)
 {
+  SjMetadataMusicbrainz5 *self;
   SjMetadataMusicbrainz5Private *priv;
   GList *albums = NULL;
   Mb5ReleaseList releases;
@@ -493,7 +698,8 @@ mb5_list_albums (SjMetadata *metadata, char **url, GError **error)
   int i;
   g_return_val_if_fail (SJ_IS_METADATA_MUSICBRAINZ5 (metadata), NULL);
 
-  priv = GET_PRIVATE (metadata);
+  self = SJ_METADATA_MUSICBRAINZ5 (metadata);
+  priv = GET_PRIVATE (SJ_METADATA_MUSICBRAINZ5 (self));
 
   if (sj_metadata_helper_check_media (priv->cdrom, error) == FALSE) {
     return NULL;
@@ -502,7 +708,7 @@ mb5_list_albums (SjMetadata *metadata, char **url, GError **error)
   priv->disc = discid_new ();
   if (priv->disc == NULL)
     return NULL;
-  if (discid_read (priv->disc, priv->cdrom) == 0)
+  if (discid_read_sparse (priv->disc, priv->cdrom, 0) == 0)
     return NULL;
 
   if (url != NULL)
@@ -516,8 +722,10 @@ mb5_list_albums (SjMetadata *metadata, char **url, GError **error)
 
   releases = mb5_query_lookup_discid(priv->mb, discid);
 
-  if (releases == NULL)
+  if (releases == NULL) {
+    print_musicbrainz_query_error (self, "discid", discid);
     return NULL;
+  }
 
   if (mb5_release_list_size (releases) == 0)
     return NULL;
@@ -530,19 +738,16 @@ mb5_list_albums (SjMetadata *metadata, char **url, GError **error)
       char *releaseid = NULL;
       Mb5Release full_release = NULL;
       Mb5Metadata release_md = NULL;
-      const char *query_entity = "release";
-      char *params_names[] = { "inc" };
-      char *params_values[] = { "artists artist-credits labels recordings \
+      char *includes = "aliases artists artist-credits labels recordings \
 release-groups url-rels discids recording-level-rels work-level-rels work-rels \
-artist-rels" };
+artist-rels";
 
       releaseid = NULL;
       GET(releaseid, mb5_release_get_id, release);
-      /* Inorder to get metadata from work and composer relationships
-       * we need to perform a custom query
+      /* Inorder to get metadata for artist aliases & work / composer
+       * relationships we need to perform a custom query
        */
-      release_md = mb5_query_query (priv->mb, query_entity, releaseid, "", 1,
-                                    params_names, params_values);
+      release_md = query_musicbrainz (self, "release", releaseid, includes);
 
       if (release_md && mb5_metadata_get_release (release_md))
         full_release = mb5_metadata_get_release (release_md);
@@ -561,12 +766,10 @@ artist-rels" };
            * release-group, so run a separate query to get these urls
            */
           char *releasegroupid = NULL;
-          char *params_names[] = { "inc" };
-          char *params_values[] = { "artists url-rels" };
+          char *includes = "artists url-rels";
 
           GET (releasegroupid, mb5_releasegroup_get_id, group);
-          metadata = mb5_query_query (priv->mb, "release-group", releasegroupid, "",
-                                      1, params_names, params_values);
+          metadata = query_musicbrainz (self, "release-group", releasegroupid, includes);
           g_free (releasegroupid);
         }
 
@@ -578,7 +781,7 @@ artist-rels" };
           Mb5Medium medium;
           medium = mb5_medium_list_item (media, j);
           if (medium) {
-            album = make_album_from_release (group, full_release, medium);
+            album = make_album_from_release (self, group, full_release, medium);
             album->metadata_source = SOURCE_MUSICBRAINZ;
             albums = g_list_append (albums, album);
           }
@@ -593,6 +796,28 @@ artist-rels" };
   return albums;
 }
 
+static void
+setup_http_proxy (SjMetadataMusicbrainz5Private *priv)
+{
+  if (priv->proxy_host == NULL || priv->proxy_port == 0) {
+    mb5_query_set_proxyhost (priv->mb, NULL);
+    mb5_query_set_proxyport (priv->mb, 0);
+    mb5_query_set_proxyusername (priv->mb, NULL);
+    mb5_query_set_proxypassword (priv->mb, NULL);
+  } else {
+    mb5_query_set_proxyhost (priv->mb, priv->proxy_host);
+    mb5_query_set_proxyport (priv->mb, priv->proxy_port);
+    if (priv->proxy_use_authentication &&
+        priv->proxy_username != NULL && priv->proxy_password != NULL) {
+      mb5_query_set_proxyusername (priv->mb, priv->proxy_username);
+      mb5_query_set_proxypassword (priv->mb, priv->proxy_password);
+    } else {
+      mb5_query_set_proxyusername (priv->mb, NULL);
+      mb5_query_set_proxypassword (priv->mb, NULL);
+    }
+  }
+}
+
 /*
  * GObject methods
  */
@@ -608,52 +833,13 @@ metadata_interface_init (gpointer g_iface, gpointer iface_data)
 static void
 sj_metadata_musicbrainz5_init (SjMetadataMusicbrainz5 *self)
 {
-  GConfClient *gconf_client;
-  gchar *server_name;
-
   SjMetadataMusicbrainz5Private *priv;
 
   priv = GET_PRIVATE (self);
+  priv->mb = mb5_query_new (SJ_MUSICBRAINZ_USER_AGENT, NULL, 0);
 
-  gconf_client = gconf_client_get_default ();
-
-  server_name = gconf_client_get_string (gconf_client, GCONF_MUSICBRAINZ_SERVER, NULL);
-
-  if (server_name && (*server_name == '\0')) {
-    g_free (server_name);
-    server_name = NULL;
-  }
-
-  priv->mb = mb5_query_new (SJ_MUSICBRAINZ_USER_AGENT, server_name, 0);
-  g_free (server_name);
-
-
-  /* Set the HTTP proxy */
-  if (gconf_client_get_bool (gconf_client, GCONF_PROXY_USE_PROXY, NULL)) {
-    char *proxy_host;
-    int port;
-
-    proxy_host = gconf_client_get_string (gconf_client, GCONF_PROXY_HOST, NULL);
-    mb5_query_set_proxyhost (priv->mb, proxy_host);
-    g_free (proxy_host);
-
-    port = gconf_client_get_int (gconf_client, GCONF_PROXY_PORT, NULL);
-    mb5_query_set_proxyport (priv->mb, port);
-
-    if (gconf_client_get_bool (gconf_client, GCONF_PROXY_USE_AUTHENTICATION, NULL)) {
-      char *username, *password;
-
-      username = gconf_client_get_string (gconf_client, GCONF_PROXY_USERNAME, NULL);
-      mb5_query_set_proxyusername (priv->mb, username);
-      g_free (username);
-
-      password = gconf_client_get_string (gconf_client, GCONF_PROXY_PASSWORD, NULL);
-      mb5_query_set_proxypassword (priv->mb, password);
-      g_free (password);
-    }
-  }
-
-  g_object_unref (gconf_client);
+  priv->artist_cache = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                              NULL, artist_details_destroy);
 }
 
 static void
@@ -667,11 +853,20 @@ sj_metadata_musicbrainz5_get_property (GObject *object, guint property_id,
   case PROP_DEVICE:
     g_value_set_string (value, priv->cdrom);
     break;
+  case PROP_PROXY_USE_AUTHENTICATION:
+    g_value_set_boolean (value, priv->proxy_use_authentication);
+    break;
   case PROP_PROXY_HOST:
-    g_value_set_string (value, priv->http_proxy);
+    g_value_set_string (value, priv->proxy_host);
     break;
   case PROP_PROXY_PORT:
-    g_value_set_int (value, priv->http_proxy_port);
+    g_value_set_int (value, priv->proxy_port);
+    break;
+  case PROP_PROXY_USERNAME:
+    g_value_set_string (value, priv->proxy_username);
+    break;
+  case PROP_PROXY_PASSWORD:
+    g_value_set_string (value, priv->proxy_password);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -691,17 +886,28 @@ sj_metadata_musicbrainz5_set_property (GObject *object, guint property_id,
       g_free (priv->cdrom);
     priv->cdrom = g_value_dup_string (value);
     break;
+  case PROP_PROXY_USE_AUTHENTICATION:
+    priv->proxy_use_authentication = g_value_get_boolean (value);
+    setup_http_proxy (priv);
+    break;
   case PROP_PROXY_HOST:
-    if (priv->http_proxy) {
-      g_free (priv->http_proxy);
-    }
-    priv->http_proxy = g_value_dup_string (value);
-    /* TODO: check this unsets the proxy if NULL, or should we pass "" ? */
-    mb5_query_set_proxyhost (priv->mb, priv->http_proxy);
+    g_free (priv->proxy_host);
+    priv->proxy_host = g_value_dup_string (value);
+    setup_http_proxy (priv);
     break;
   case PROP_PROXY_PORT:
-    priv->http_proxy_port = g_value_get_int (value);
-    mb5_query_set_proxyport (priv->mb, priv->http_proxy_port);
+    priv->proxy_port = g_value_get_int (value);
+    setup_http_proxy (priv);
+    break;
+  case PROP_PROXY_USERNAME:
+    g_free (priv->proxy_username);
+    priv->proxy_username = g_value_dup_string (value);
+    setup_http_proxy (priv);
+    break;
+  case PROP_PROXY_PASSWORD:
+    g_free (priv->proxy_password);
+    priv->proxy_password = g_value_dup_string (value);
+    setup_http_proxy (priv);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -720,10 +926,14 @@ sj_metadata_musicbrainz5_finalize (GObject *object)
     priv->mb = NULL;
   }
   if (priv->disc != NULL) {
-      discid_free (priv->disc);
-      priv->disc = NULL;
+    discid_free (priv->disc);
+    priv->disc = NULL;
   }
   g_free (priv->cdrom);
+  g_free (priv->proxy_host);
+  g_free (priv->proxy_username);
+  g_free (priv->proxy_password);
+  g_hash_table_unref (priv->artist_cache);
 
   G_OBJECT_CLASS (sj_metadata_musicbrainz5_parent_class)->finalize (object);
 }
@@ -731,6 +941,7 @@ sj_metadata_musicbrainz5_finalize (GObject *object)
 static void
 sj_metadata_musicbrainz5_class_init (SjMetadataMusicbrainz5Class *class)
 {
+  const gchar * const * l;
   GObjectClass *object_class = (GObjectClass*)class;
 
   g_type_class_add_private (class, sizeof (SjMetadataMusicbrainz5Private));
@@ -739,11 +950,33 @@ sj_metadata_musicbrainz5_class_init (SjMetadataMusicbrainz5Class *class)
   object_class->set_property = sj_metadata_musicbrainz5_set_property;
   object_class->finalize = sj_metadata_musicbrainz5_finalize;
 
-  g_object_class_override_property (object_class, PROP_DEVICE, "device");
-  g_object_class_override_property (object_class, PROP_PROXY_HOST, "proxy-host");
-  g_object_class_override_property (object_class, PROP_PROXY_PORT, "proxy-port");
-}
+  g_object_class_override_property (object_class,
+                                    PROP_DEVICE, "device");
+  g_object_class_override_property (object_class,
+                                    PROP_PROXY_USE_AUTHENTICATION, "proxy-use-authentication");
+  g_object_class_override_property (object_class,
+                                    PROP_PROXY_HOST, "proxy-host");
+  g_object_class_override_property (object_class,
+                                    PROP_PROXY_PORT, "proxy-port");
+  g_object_class_override_property (object_class,
+                                    PROP_PROXY_USERNAME, "proxy-username");
+  g_object_class_override_property (object_class,
+                                    PROP_PROXY_PASSWORD, "proxy-password");
 
+  /* Although GLib supports fallback locales we do not use them as
+     MusicBrainz does not provide locale information for the canonical
+     artist name, only it's aliases. This means that it is not
+     possible to fallback to other locales as the canonical name may
+     actually be in a higher priority locale than the alias that
+     matches one of the fallback locales. */
+  l = g_get_language_names ();
+  if (strcmp (l[0], "C") == 0) { /* "C" locale should be "en" for MusicBrainz */
+    memcpy (language, "en", 3);
+  } else if (strlen (l[0]) > 1) {
+    memcpy (language, l[0], 2);
+    language[2] = '\0';
+  }
+}
 
 /*
  * Public methods.
