@@ -14,9 +14,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.*
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.*
  *
  * Authors: Ross Burton <ross@burtonini.com>
  */
@@ -37,8 +35,6 @@ void track_details_free(TrackDetails *track)
   g_free (track->track_id);
   g_free (track->artist_id);
   g_free (track->artist_sortname);
-  g_list_foreach (track->artists, (GFunc)artist_details_free, NULL);
-  g_list_free (track->artists);
   g_free (track);
 }
 
@@ -65,8 +61,7 @@ void album_details_free(AlbumDetails *album)
   g_free (album->lyrics_url);
   g_free (album->country);
   g_free (album->type);
-  g_list_foreach (album->artists, (GFunc)artist_details_free, NULL);
-  g_list_free (album->artists);
+  g_list_foreach (album->labels, (GFunc)label_details_free, NULL);
   g_free (album);
 }
 
@@ -81,6 +76,42 @@ void artist_details_free (ArtistDetails *artist)
   g_free (artist->disambiguation);
   g_free (artist->gender);
   g_free (artist->country);
-  g_free (artist->joinphrase);
   g_free (artist);
 }
+
+/*
+ * GDestroyNotify callback to free a ArtistDetails*
+ */
+void artist_details_destroy (gpointer artist)
+{
+  artist_details_free (artist);
+}
+
+/*
+ * Free an ArtistCredit*
+ */
+void artist_credit_free (ArtistCredit *credit, gboolean free_details)
+{
+  if (free_details)
+    artist_details_free (credit->details);
+  g_free (credit->joinphrase);
+  g_slice_free (ArtistCredit, credit);
+}
+
+/*
+ * GDestroyNotify callback to free a ArtistCredit*
+ */
+void artist_credit_destroy (gpointer artist)
+{
+  artist_credit_free (artist, FALSE);
+}
+
+/*
+ * Free a LabelDetails
+ */
+void label_details_free (LabelDetails *label)
+{
+  g_free (label->name);
+  g_free (label->sortname);
+  g_free (label);
+}
