@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Ross Burton <ross@burtonini.com>
  */
@@ -41,6 +40,7 @@
 
 #include "sj-error.h"
 #include "sj-extracting.h"
+#include "sj-main.h"
 #include "sj-util.h"
 #include "sj-play.h"
 #include "sj-inhibit.h"
@@ -85,10 +85,7 @@ static gboolean successful_extract = FALSE;
 static GtkWidget *progress_bar, *status_bar;
 
 /** The widgets in the main UI */
-static GtkWidget *extract_button, *play_button, *title_entry, *artist_entry, *genre_entry, *year_entry, *disc_number_entry, *track_listview;
-
-/** The menuitem in the main menu */
-static GtkWidget *extract_menuitem, *play_menuitem, *reread_menuitem, *select_all_menuitem, *deselect_all_menuitem;
+static GtkWidget *extract_button, *play_button, *title_entry, *artist_entry, *composer_entry, *genre_entry, *year_entry, *disc_number_entry, *track_listview;
 
 static GtkTreeIter current;
 
@@ -232,9 +229,8 @@ cleanup (void)
   }
   /* Forcibly invalidate the iterator */
   current.stamp = 0;
-  /* TODO: find out why GTK+ needs this to work (see #364371) */
-  gtk_button_set_label (GTK_BUTTON (extract_button), _("Extract"));
-  gtk_button_set_label (GTK_BUTTON (extract_button), SJ_STOCK_EXTRACT);
+
+  gtk_button_set_label (GTK_BUTTON (extract_button), _("E_xtract"));
 
   /* Clear the Status bar */
   gtk_statusbar_push (GTK_STATUSBAR (status_bar), 0, "");
@@ -245,15 +241,15 @@ cleanup (void)
   gtk_widget_set_sensitive (play_button, TRUE);
   gtk_widget_set_sensitive (title_entry, TRUE);
   gtk_widget_set_sensitive (artist_entry, TRUE);
+  gtk_widget_set_sensitive (composer_entry, TRUE);
   gtk_widget_set_sensitive (genre_entry, TRUE);
   gtk_widget_set_sensitive (year_entry, TRUE);
   gtk_widget_set_sensitive (disc_number_entry, TRUE);
   /* Enabling the Menuitem */
-  gtk_widget_set_sensitive (play_menuitem, TRUE);
-  gtk_widget_set_sensitive (extract_menuitem, TRUE);
-  gtk_widget_set_sensitive (reread_menuitem, TRUE);
-  gtk_widget_set_sensitive (select_all_menuitem, TRUE);
-  gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
+  set_action_enabled ("play", TRUE);
+  set_action_enabled ("select-all", TRUE);
+  set_action_enabled ("deselect-all", TRUE);
+  set_action_enabled ("re-read", TRUE);
 
   /*Enable the Extract column and Make the Title and Artist column Editable*/
   g_object_set (G_OBJECT (toggle_renderer), "mode", GTK_CELL_RENDERER_MODE_ACTIVATABLE, NULL);
@@ -458,7 +454,7 @@ pop_and_extract (int *overwrite_mode)
     /* OK, we can write/overwrite the file */
 
 
-    /* Update the state stock image */
+    /* Update the state image */
     gtk_list_store_set (track_store, &current,
                    COLUMN_STATE, STATE_EXTRACTING, -1);
 
@@ -746,7 +742,7 @@ on_progress_cancel_clicked (GtkWidget *button, gpointer user_data)
 G_MODULE_EXPORT void
 on_extract_activate (GtkWidget *button, gpointer user_data)
 {
-  char *reason;
+  char *reason = NULL;
 
   /* first make sure we're not playing, we cannot share the resource */
   stop_playback ();
@@ -782,6 +778,7 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
     play_button       = GET_WIDGET ("play_button");
     title_entry       = GET_WIDGET ("title_entry");
     artist_entry      = GET_WIDGET ("artist_entry");
+    composer_entry    = GET_WIDGET ("composer_entry");
     genre_entry       = GET_WIDGET ("genre_entry");
     year_entry        = GET_WIDGET ("year_entry");
     disc_number_entry = GET_WIDGET ("disc_number_entry");
@@ -789,19 +786,11 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
     progress_bar      = GET_WIDGET ("progress_bar");
     status_bar        = GET_WIDGET ("status_bar");
 
-    play_menuitem         = GET_WIDGET ("play_menuitem");
-    extract_menuitem      = GET_WIDGET ("extract_menuitem");
-    reread_menuitem       = GET_WIDGET ("re-read");
-    select_all_menuitem   = GET_WIDGET ("select_all");
-    deselect_all_menuitem = GET_WIDGET ("deselect_all");
-
     initialised = TRUE;
   }
 
   /* Change the label to Stop while extracting*/
-  /* TODO: find out why GTK+ needs this to work (see #364371) */
-  gtk_button_set_label (GTK_BUTTON (extract_button), _("Stop"));
-  gtk_button_set_label (GTK_BUTTON (extract_button), GTK_STOCK_STOP);
+  gtk_button_set_label (GTK_BUTTON (extract_button), _("_Stop"));
   gtk_widget_show (progress_bar);
 
   /* Reset the progress dialog */
@@ -812,16 +801,16 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   gtk_widget_set_sensitive (play_button, FALSE);
   gtk_widget_set_sensitive (title_entry, FALSE);
   gtk_widget_set_sensitive (artist_entry, FALSE);
+  gtk_widget_set_sensitive (composer_entry, FALSE);
   gtk_widget_set_sensitive (genre_entry, FALSE);
   gtk_widget_set_sensitive (year_entry, FALSE);
   gtk_widget_set_sensitive (disc_number_entry, FALSE);
 
   /* Disable the menuitems in the main menu*/
-  gtk_widget_set_sensitive (play_menuitem, FALSE);
-  gtk_widget_set_sensitive (extract_menuitem, FALSE);
-  gtk_widget_set_sensitive (reread_menuitem, FALSE);
-  gtk_widget_set_sensitive (select_all_menuitem, FALSE);
-  gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
+  set_action_enabled ("play", FALSE);
+  set_action_enabled ("select-all", FALSE);
+  set_action_enabled ("deselect-all", FALSE);
+  set_action_enabled ("re-read", FALSE);
 
   /* Disable the Extract column */
   g_object_set (G_OBJECT (toggle_renderer), "mode", GTK_CELL_RENDERER_MODE_INERT, NULL);
@@ -869,6 +858,8 @@ sanitize_path (const char* str, const char* filesystem_type)
   gchar *res = NULL;
   gchar *s;
 
+  g_return_val_if_fail (str != NULL, NULL);
+
   /* Skip leading periods, otherwise files disappear... */
   while (*str == '.')
     str++;
@@ -899,6 +890,56 @@ sanitize_path (const char* str, const char* filesystem_type)
   return res ? res : g_strdup(str);
 }
 
+/*
+ * Return sanitized name or default if name is empty
+ */
+static char*
+sanitize_name (const char *name, const char *default_name,
+               const char *filesystem_type)
+{
+  const char *n = (sj_str_is_empty (name)) ? default_name : name;
+  return sanitize_path (n, filesystem_type);
+}
+
+/*
+ * Return lowercase sanitized name or default if name is empty
+ */
+static char*
+lower_sanitize_name (const char *name, const char *default_name,
+                     const char *filesystem_type)
+{
+  char *tmp, *s;
+  const char *n = (sj_str_is_empty (name)) ? default_name : name;
+  tmp = g_utf8_strdown (n, -1);
+  s = sanitize_path (tmp, filesystem_type);
+  g_free (tmp);
+  return s;
+}
+
+/*
+ * Return sanitized sortname or name if sortname is empty or default
+ * if name is empty
+ */
+static char*
+sanitize_sortname (const char *sortname, const char *name,
+                   const char *default_name, const char *filesystem_type)
+{
+  const char *n = (sj_str_is_empty (sortname)) ? name : sortname;
+  return sanitize_name (n, default_name, filesystem_type);
+}
+
+/*
+ * Return lowercase sanitized sortname or name if sortname is empty or
+ * default if name is empty
+ */
+static char*
+lower_sanitize_sortname (const char *sortname, const char *name,
+                         const char *default_name, const char *filesystem_type)
+{
+  const char *n = (sj_str_is_empty (sortname)) ? name : sortname;
+  return lower_sanitize_name (n, default_name, filesystem_type);
+}
+
 /**
  * Parse a filename pattern and replace markers with values from a TrackDetails
  * structure.
@@ -910,6 +951,10 @@ sanitize_path (const char* str, const char* filesystem_type)
  * %aA -- album artist (lowercase)
  * %as -- album artist sortname
  * %aS -- album artist sortname (lowercase)
+ * %ac -- album composer
+ * %aC -- album composer (lowercase)
+ * %ap -- album composer (sortable)
+ * %aP -- album composer (sortable lowercase)
  * %tn -- track number (i.e 8)
  * %tN -- track number, zero padded (i.e 08)
  * %tt -- track title
@@ -918,8 +963,12 @@ sanitize_path (const char* str, const char* filesystem_type)
  * %tA -- track artist (lowercase)
  * %ts -- track artist sortname
  * %tS -- track artist sortname (lowercase)
- * %dn -- disc and track number (i.e Disk 2 - 6, or 6)
- * %dN -- disc number, zero padded (i.e d02t06, or 06)
+ * %tc -- track composer
+ * %tC -- track composer (lowercase)
+ * %tp -- track composer (sortable)
+ * %tP -- track composer (sortable lowercase)
+ * %dn -- disc and track number, track zero padded (i.e Disk 2 - 06, or 06)
+ * %dN -- condensed disc and track number, zero padded (i.e d02t06, or 06)
  */
 char*
 filepath_parse_pattern (const char* pattern, const TrackDetails *track)
@@ -929,6 +978,10 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
   char *tmp, *str, *filesystem_type = NULL;
   GString *s;
   GFileInfo *fs_info;
+  const char *default_album    = _("Unknown Album");
+  const char *default_artist   = _("Unknown Artist");
+  const char *default_composer = _("Unknown Composer");
+  const char *default_track    = _("Unknown Track");
 
   if (pattern == NULL || pattern[0] == 0)
     return g_strdup (" ");
@@ -973,7 +1026,8 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        string = sanitize_path (track->album->title, filesystem_type);
+        string = sanitize_name (track->album->title, default_album,
+                                filesystem_type);
         break;
       case 'y':
         if (track->album->release_date && g_date_valid(track->album->release_date)) {
@@ -983,25 +1037,44 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
         }
         break;
       case 'T':
-        tmp = g_utf8_strdown (track->album->title, -1);
-        string = sanitize_path (tmp, filesystem_type);
-        g_free (tmp);
+        string = lower_sanitize_name (track->album->title, default_album,
+                                      filesystem_type);
         break;
       case 'a':
-        string = sanitize_path (track->album->artist, filesystem_type);
+        string = sanitize_name (track->album->artist, default_artist,
+                                filesystem_type);
         break;
       case 'A':
-        tmp = g_utf8_strdown (track->album->artist, -1);
-        string = sanitize_path (tmp, filesystem_type);
-        g_free (tmp);
+        string = lower_sanitize_name (track->album->artist, default_artist,
+                                      filesystem_type);
         break;
       case 's':
-        string = sanitize_path (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, filesystem_type);
+        string = sanitize_sortname (track->album->artist_sortname,
+                                    track->album->artist, default_artist,
+                                    filesystem_type);
         break;
       case 'S':
-        tmp = g_utf8_strdown (track->album->artist_sortname ? track->album->artist_sortname : track->album->artist, -1);
-        string = sanitize_path (tmp, filesystem_type);
-        g_free(tmp);
+        string = lower_sanitize_sortname (track->album->artist_sortname,
+                                          track->album->artist, default_artist,
+                                          filesystem_type);
+        break;
+      case 'c':
+        string = sanitize_name (track->album->composer, default_composer,
+                                filesystem_type);
+        break;
+      case 'C':
+        string = lower_sanitize_name (track->album->composer, default_composer,
+                                      filesystem_type);
+        break;
+      case 'p':
+        string = sanitize_sortname (track->album->composer_sortname,
+                                    track->album->composer, default_composer,
+                                    filesystem_type);
+        break;
+      case 'P':
+        string = lower_sanitize_sortname (track->album->composer_sortname,
+                                          track->album->composer,
+                                          default_composer, filesystem_type);
         break;
       default:
         /* append "%a", and then the unicode character */
@@ -1019,28 +1092,43 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
        */
       switch (*++p) {
       case 't':
-        string = sanitize_path (track->title, filesystem_type);
+        string = sanitize_name (track->title, default_track, filesystem_type);
         break;
       case 'T':
-        tmp = g_utf8_strdown (track->title, -1);
-        string = sanitize_path (tmp, filesystem_type);
-        g_free(tmp);
+        string = lower_sanitize_name (track->title, default_track,
+                                      filesystem_type);
         break;
       case 'a':
-        string = sanitize_path (track->artist, filesystem_type);
+        string = sanitize_name (track->artist, default_artist, filesystem_type);
         break;
       case 'A':
-        tmp = g_utf8_strdown (track->artist, -1);
-        string = sanitize_path (tmp, filesystem_type);
-        g_free(tmp);
+        string = lower_sanitize_name (track->artist, default_artist,
+                                      filesystem_type);
         break;
       case 's':
-        string = sanitize_path (track->artist_sortname ? track->album->artist_sortname : track->artist, filesystem_type);
+        string = sanitize_sortname (track->artist_sortname, track->artist,
+                                    default_artist, filesystem_type);
         break;
       case 'S':
-        tmp = g_utf8_strdown (track->artist_sortname ? track->album->artist_sortname : track->artist, -1);
-        string = sanitize_path (tmp, filesystem_type);
-        g_free(tmp);
+        string = lower_sanitize_sortname (track->artist_sortname, track->artist,
+                                          default_artist, filesystem_type);
+        break;
+      case 'c':
+         string = sanitize_name (track->composer, default_composer,
+                                 filesystem_type);
+        break;
+      case 'C':
+        string = lower_sanitize_name (track->composer, default_composer,
+                                      filesystem_type);
+        break;
+      case 'p':
+        string = sanitize_sortname (track->composer_sortname, track->composer,
+                                    default_composer, filesystem_type);
+        break;
+      case 'P':
+        string = lower_sanitize_sortname (track->composer_sortname,
+                                          track->composer, default_composer,
+                                          filesystem_type);
         break;
       case 'n':
         /* Track number */
@@ -1068,7 +1156,9 @@ filepath_parse_pattern (const char* pattern, const TrackDetails *track)
       case 'n':
         /* Disc and track number */
         if (track->album->disc_number > 0) {
-          string = g_strdup_printf ("Disc %d - %d", track->album->disc_number, track->number);
+          char *s = g_strdup_printf ("Disc %d - %02d", track->album->disc_number, track->number);
+          string = sanitize_path (s, filesystem_type); /* strip spaces if required */
+          g_free (s);
         } else {
           string = g_strdup_printf ("%d", track->number);
         }
