@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Ronald S. Bultje <rbultje@ronald.bitfreak.net>
  */
@@ -29,6 +28,7 @@
 
 #include "sound-juicer.h"
 #include "sj-play.h"
+#include "sj-main.h"
 
 static GstElement *pipeline = NULL;
 static guint id = 0, button_change_id = 0;
@@ -40,7 +40,7 @@ static void set_gst_ui_and_play (void);
 
 static GtkTreeIter current_iter;
 
-static GtkWidget *play_button, *next_menuitem, *prev_menuitem, *seek_scale, *volume_button, *statusbar,  *track_listview;
+static GtkWidget *play_button, *seek_scale, *volume_button, *statusbar,  *track_listview;
 
 /**
  * Select track number.
@@ -86,8 +86,8 @@ _play (void)
 {
   gst_element_set_state (pipeline, GST_STATE_PLAYING);
 
-  gtk_widget_set_sensitive (next_menuitem, TRUE);
-  gtk_widget_set_sensitive (prev_menuitem, TRUE);
+  set_action_enabled ("next-track", TRUE);
+  set_action_enabled ("previous-track", TRUE);
 }
 
 /**
@@ -109,13 +109,16 @@ _stop (void)
     gst_element_set_state (pipeline, GST_STATE_NULL);
 
   /* TODO: this should be centralised into the state change logic really */
-  gtk_widget_set_sensitive (next_menuitem, FALSE);
-  gtk_widget_set_sensitive (prev_menuitem, FALSE);
+  set_action_enabled ("next-track", FALSE);
+  set_action_enabled ("previous-track", FALSE);
+
+  set_action_enabled ("re-read", TRUE);
+
   gtk_widget_hide (seek_scale);
   gtk_widget_hide (volume_button);
   sj_main_set_title (NULL);
   gtk_statusbar_push (GTK_STATUSBAR (statusbar), 0, "");
-  gtk_button_set_label (GTK_BUTTON (play_button), GTK_STOCK_MEDIA_PLAY);
+  gtk_button_set_label (GTK_BUTTON (play_button), _("_Play"));
   slen = GST_CLOCK_TIME_NONE;
 
   if (current_track != -1 &&
@@ -253,9 +256,7 @@ cb_change_button (gpointer data)
 {
   button_change_id = 0;
 
-  /* TODO: find out why GTK+ needs this to work (see #364371) */
-  gtk_button_set_label (GTK_BUTTON (data), _("Play"));
-  gtk_button_set_label (GTK_BUTTON (data), GTK_STOCK_MEDIA_PLAY);
+  gtk_button_set_label (GTK_BUTTON (data), _("_Play"));
 
   /* once */
   return FALSE;
@@ -297,9 +298,7 @@ cb_state (GstBus *bus, GstMessage *message, gpointer user_data)
     gtk_statusbar_push (GTK_STATUSBAR (statusbar), 0, "");
     current_track = -1;
   } else if (transition == GST_STATE_CHANGE_PAUSED_TO_PLAYING) {
-    /* TODO: find out why GTK+ needs this to work (see #364371) */
-    gtk_button_set_label (GTK_BUTTON (play_button), _("Pause"));
-    gtk_button_set_label (GTK_BUTTON (play_button), GTK_STOCK_MEDIA_PAUSE);
+    gtk_button_set_label (GTK_BUTTON (play_button), _("_Pause"));
     if (id)
       g_source_remove (id);
     id = g_timeout_add (100, (GSourceFunc) cb_set_time, NULL);
@@ -386,9 +385,7 @@ stop_playback (void)
 /*
  * Interface entry point.
  */
-
-G_MODULE_EXPORT void
-on_play_activate (GtkWidget *button, gpointer user_data)
+void toggle_play ()
 {
   GError *err = NULL;
 
@@ -452,14 +449,12 @@ on_tracklist_row_activate (GtkTreeView * treeview, GtkTreePath * path,
   }
 }
 
-G_MODULE_EXPORT void
-on_next_track_activate(GtkWidget *button, gpointer data)
+void play_next_track ()
 {
   cb_hop_track (NULL, NULL, NULL);
 }
 
-G_MODULE_EXPORT void
-on_previous_track_activate(GtkWidget *button, gpointer data)
+void play_previous_track ()
 {
   GtkTreeModel *model;
   gint prev_track = current_track - 1;
@@ -536,7 +531,21 @@ on_volume_changed (GtkWidget * volb, gdouble value, gpointer data)
   vol = value;
   if (pipeline)
     g_object_set (G_OBJECT (pipeline), "volume", vol, NULL);
-  gconf_client_set_float (gconf_client, GCONF_AUDIO_VOLUME, vol, NULL);
+  g_settings_set_double (sj_settings, SJ_SETTINGS_AUDIO_VOLUME, vol);
+}
+
+static gboolean
+is_non_seek_key (GdkEvent * event)
+{
+  guint key;
+
+  return gdk_event_get_keyval (event, &key) &&
+    key != GDK_KEY_Left  && key != GDK_KEY_KP_Left  &&
+    key != GDK_KEY_Right && key != GDK_KEY_KP_Right &&
+    key != GDK_KEY_Up    && key != GDK_KEY_KP_Up    &&
+    key != GDK_KEY_Down  && key != GDK_KEY_KP_Down  &&
+    key != GDK_KEY_End   && key != GDK_KEY_KP_End   &&
+    key != GDK_KEY_Home  && key != GDK_KEY_KP_Home;
 }
 
 /*
@@ -544,8 +553,11 @@ on_volume_changed (GtkWidget * volb, gdouble value, gpointer data)
  */
 
 G_MODULE_EXPORT gboolean
-on_seek_press (GtkWidget * scale, GdkEventButton * event, gpointer user_data)
+on_seek_press (GtkWidget * scale, GdkEvent * event, gpointer user_data)
 {
+  if (is_non_seek_key (event))
+    return FALSE;
+
   seeking = TRUE;
 
   return FALSE;
@@ -568,10 +580,15 @@ on_seek_moved (GtkWidget * scale, gpointer user_data)
 }
 
 G_MODULE_EXPORT gboolean
-on_seek_release (GtkWidget * scale, GdkEventButton * event, gpointer user_data)
+on_seek_release (GtkWidget * scale, GdkEvent * event, gpointer user_data)
 {
   gdouble val = gtk_range_get_value (GTK_RANGE (scale));
 
+  /* If gst_element_seek is called when non-seeking key is released it
+     causes a glitch in playback*/
+  if (is_non_seek_key (event))
+    return FALSE;
+
   seeking = FALSE;
 
   gst_element_seek (pipeline, 1.0, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, slen * val, GST_SEEK_TYPE_NONE, -1);
@@ -585,7 +602,7 @@ on_seek_release (GtkWidget * scale, GdkEventButton * event, gpointer user_data)
 void
 stop_ui_hack (void)
 {
-  gtk_button_set_label (GTK_BUTTON (play_button), GTK_STOCK_MEDIA_PLAY);
+  gtk_button_set_label (GTK_BUTTON (play_button), _("_Play"));
   gtk_widget_hide (seek_scale);
   gtk_widget_hide (volume_button);
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 0);
@@ -604,8 +621,6 @@ void
 sj_play_init (void)
 {
   play_button     = GET_WIDGET ("play_button");
-  next_menuitem   = GET_WIDGET ("next_track_menuitem");
-  prev_menuitem   = GET_WIDGET ("previous_track_menuitem");
   seek_scale      = GET_WIDGET ("seek_scale");
   volume_button   = GET_WIDGET ("volume_button");
   statusbar       = GET_WIDGET ("status_bar");
